function bounds=bounds(q1,q0,t,fosd,graph)
% Find max and min of the probability of Y(1)<=Y(0)+t
% inputs:   q1: quantiles of Y(1) ((n+1)x1 vector)
%           q0: quantiles of Y(0) ((n+1)x1 vector)
%           t:  treatment effect size (scalar)
%           fosd: 2x1 vector for imposing stochastic increasingness
%               [0,0]: no stochastic increasing constraint
%               [1,0]: Y(1) stochastically increasing in Y(0)
%               [0,1]: Y(0) stochastically increasing in Y(1)
%               [1,1]: each SI in the other
%           n: size of grid
% output: [lb;ub] lower and upper bound for probability

%% Make f
n=length(q1);
below = zeros(n*n,1);
for pp = 1:n % pp indexes q0
    for kk = 1:n % kk indexes q1
    below(kk+(pp-1)*n,1) = (q1(kk)<=(q0(pp)+t));
    end
end

%% Stochastic Ordering Requirements 

% Impose FOSD
Aineq1=[];
bineq1=[];
Aineq2=[];
bineq2=[];
if sum(fosd)>0
    differfactor1=[diag(-ones(n-1,1))+diag(ones(n-2,1),1),[zeros(1,n-2),1]'];
    differfactor2=[eye(n-1),zeros(n-1,1)];
    if fosd(1)>0
        cdfer1=kron(eye(n),tril(ones(n,n)));
        differ1=kron(differfactor1,differfactor2);
        Aineq1=differ1*cdfer1;
        bineq1=zeros((n-1)^2,1);
    end
    if fosd(2)>0
        cdfer2=kron(tril(ones(n,n)),eye(n));
        differ2=kron(differfactor2,differfactor1);
        Aineq2=differ2*cdfer2;
        bineq2=zeros((n-1)^2,1);
    end
    
end
Aineq=[Aineq1;Aineq2];
bineq=[bineq1;bineq2];


%% Adding-up Constraints

Aeq1 = kron(eye(n),ones(1,n)); %each column sums to 1/n
Aeq2 = kron(ones(1,n),differfactor2); % each row sums to 1/n, but nth row
% is redundant given Aeq1

Aeq = [Aeq1 ; Aeq2];
beq = (1/n)*ones(2*n-1,1);

lb = zeros(n^2,1);
ub = ones(n^2,1)./n;

%% Copula maximization 
options = optimoptions('linprog','Algorithm','dual-simplex','MaxIter',10000000);
    
if sum(fosd) == 0
    para_start = ones(n^2,1)/(n^2);
    [best_cop,est_min_val] = linprog(below,[],[],Aeq,beq,lb,ub,para_start,options);
    [worst_cop,est_max_val] = linprog(-below,[],[],Aeq,beq,lb,ub,para_start,options);
    est_max_val = -est_max_val;
else
    para_start = ones(n^2,1)/(n^2);
    [best_cop,est_min_val] = linprog(below,Aineq,bineq,Aeq,beq,lb,ub,para_start,options);
    [worst_cop,est_max_val] = linprog(-below,Aineq,bineq,Aeq,beq,lb,ub,para_start,options);
    est_max_val = -est_max_val;
end

bounds=[est_min_val;est_max_val]

if graph>0
    %Plot results
    copulae=figure('Position',[520 378 1120 420],...
        'PaperOrientation','landscape',...
        'PaperPosition',[1,3,9,4]);
    subplot(1,2,1);
    axis([0 1 0 1]);
    subplot(1,2,2);
    axis([0 1 0 1]);
   

    for kk = n:-1:1
        for pp = 1:n
            shadew=min(max(1-n*worst_cop(kk+(pp-1)*n),0),1);
            shadeb=min(max(1-n*best_cop(kk+(pp-1)*n),0),1);
         
            line=1-below(kk+(pp-1)*n);
            figure(copulae);
            subplot(1,2,2);
            rectangle('Position',[(pp-1)/n (kk-1)/n 1/n 1/n],...
            'FaceColor',[shadew shadew shadew], ...
            'EdgeColor', [line line line]);
            subplot(1,2,1);
            rectangle('Position',[(pp-1)/n (kk-1)/n 1/n 1/n],...
            'FaceColor',[shadeb shadeb shadeb], ...
            'EdgeColor', [line line line]);
        end
    end
    figure(copulae);
    subplot(1,2,2);
    title(sprintf('Upper bound copula\nPr(Y(1)<Y(0)+%0.2g) = %0.2g\n',t,est_max_val));
    xlabel('U(0)');
    ylabel('U(1)');
    subplot(1,2,1);
    title(sprintf('Lower bound copula\nPr(Y(1)<Y(0)+%0.2g = %0.2g\n',t,est_min_val));    
    xlabel('U(0)');
    ylabel('U(1)');
end
end