/*
PROGRAM: replication_kipp.do
PROGRAMMER: Brigham Frandsen
PURPOSE: replicate results for QE paper
*/
pause on
local output replication_kipp
capture log close
log using `output'.log,replace text
clear ado
capture log close
set more off
set matsize 11000

* assumes tedistboundscicentered.ado is saved in the parent directory of Stata's current working directory
adopath + ../

* assumes KIPP data are stored in the following directory:
local data ../../KIPP archive for KIPP2/data


* mata programs to be used below

mata mata clear
mata:
	pointer vector wls(string scalar varlist,string scalar pvarlist)
	{ // does weighted least squares of depvar on x1 x2 x3 with weights kappa.
	//feed it: depvar kappa x1 x2 x3 ...
	// pvarlist is a list of variables that will be used to construct the predicted values
	
	vars=tokens(varlist)
	pvars=tokens(pvarlist)
	y=st_data(.,vars[1])
	kappa=st_data(.,vars[2])
	if (length(vars)>=3) {
		xs=st_data(.,vars[3..length(vars)])
		xs=xs,J(rows(xs),1,1)
	}
	else xs=J(rows(y),1,1)
	if (length(pvars)>=1) {
		pxs=st_data(.,pvars[1..length(pvars)])
		pxs=pxs,J(rows(pxs),1,1)
	}
	else pxs=J(rows(y),1,1)
	// set missing rows in y to be zero in kappa
	kappa=(y:!=.):*kappa
	xkx=cross(xs,kappa,xs)
	xky=cross(xs,kappa,y)
	xkxinv=invsym(xkx)
	beta=xkxinv*xky
	u=y-xs*beta
	u2k=u:^2:*kappa
	varcov=xkxinv*cross(xs,u2k,xs)*xkxinv
	yhat=pxs*beta
	return((&beta,&varcov,&yhat))
}
end



* bring in KIPP data (from the Table 4 replication file) and clean

do ../stacking_grades_bf.do
rename (MA_c_mrawsc*) (math*)
rename (MA_c_erawsc*) (english*)

local covariates  female4 black4 hisp4 asian4 otherrace4 sped4 lep4 foodfred4 fem_min4

local s1 ones
local s2 sspline*
local s3 sspline* `covariates'
local s1title Predictors: none
local s2title Predictors: baseline outcome
local s3title Predictors: baseline outcome and covariates
local exchtitle not imposed
local exchexchtitle imposed


local y1 math
local `y1'title Math score
local y10 math4


local z offer
local d ever_enrolled


local nknots 3


gen byte ones=1
gen byte zeros=0

gen minority4=black4==1 | hisp4==1 | asian4==1 | otherrace4==1
gen female4_x_minority4 = female4==1 & minority4==1


* keep only grade 6 outcomes and applicants from 2006-2007
tab grade
tab yearapp
keep if grade==6 & (yearapp==2006 | yearapp==2007)


keep `y1' `d' `z' `y10' `covariates' yearapp ones

tempfile goback
save `goback'

* The following is to estimate the bounds the impose stochastic increasingness uniformly for Figure 2 and Table 4
* Estimate quantiles of Y(0) and Y(1)  both overall and 
* compliers via 2sls, and save to data file `output'y01ranks.dta

reg `d' `z'
scalar fs=_b[`z']
gen w0=.
gen w1=.

local condall qy4!=.
local cond0 qy4==0
local cond1 qy4==1
local cond2 qy4==2
local cond3 qy4==3
local appendlist
sum `z'
gen kappa=1-`d'*(1-`z')/(1-r(mean))-(1-`d')*`z'/r(mean)
foreach y in `y1' `y2' {
	egen qy4=cut(`y'4),group(4)
	
	foreach q in 0 1 2 3 {
		display "on q = `q'"
		gen w=(qy4==`q')*kappa/fs
		sum w	
		drop w
	}
	
	foreach cond in all 0 1 2 3 {
		preserve
		keep if `cond`cond''
		matrix `y'01ranks=J(1,3,.)
		sum `y'
		local min=r(min)
		local max=r(max)
		local step=(`max'-`min')/100
		forvalues c=`min'(`step')`max' {
			qui replace w0=`y'<=`c' & `d'==0
			qui reg w0 `z'
			scalar rf=_b[`z']
			scalar coeff0=rf/fs
					
			qui replace w1=`y'<=`c' & `d'==1
			qui reg w1 `z'
			scalar rf=_b[`z']
			scalar coeff1=rf/fs
			matrix `y'01ranks=`y'01ranks\(`c',max(0,min(1,-coeff0)),max(0,min(1,coeff1)))		
		}
		clear
		matrix colnames `y'01ranks= yval rank0 rank1
		mat li `y'01ranks
		svmat `y'01ranks,names(col)
		gen yvar="`y'"
		gen y4quartile="`cond'"
		tempfile `y'01ranks`cond'
		save ``y'01ranks`cond''
		local appendlist `appendlist' ``y'01ranks`cond''
		restore	
	}
	drop qy4
}
drop w0
clear
append using `appendlist'
save `output'y01ranks.dta,replace

* prepare percentile grid of ranks:
local ngrid=50
clear
set obs `ngrid'
gen rank0=_n/`ngrid'-1/`ngrid'/2
gen rank1=rank0
gen byte forgrid=1
gen yvar="math"
gen y4quartile="0"
local numrows=0
foreach yvar in math english {
	foreach y4q in 0 1 2 3 all {
		local ++ numrows
		expand 2 if yvar=="math" & y4quartile=="0",gen(dup)
		replace yvar="`yvar'" if dup==1
		replace y4quartile="`y4q'" if dup==1
		drop dup
	}
}
duplicates drop
tempfile grid
save `grid'

use replication_kipppercentilesy01ranks if yval!=.,clear

* rearrange to make percentiles monotonic
sort yvar y4q yval
foreach yvar in math english {
	foreach y4q in 0 1 2 3 all {
		gen byte want= yvar=="`yvar'" & y4quartile=="`y4q'"
		mata r=st_data(.,"rank0","want")
		mata _sort(r,1)
		mata st_store(.,"rank0","want",r)
		mata r=st_data(.,"rank1","want")
		mata _sort(r,1)
		mata st_store(.,"rank1","want",r)
		twoway line rank0 rank1 yval if want==1
		drop want
	}
}


append using `grid'
* initialize matrix for matlab
local numcols=2*(`ngrid'+1)
matrix formatlab=J(`numrows',`numcols',.)

* first need to interpolate percentiles onto fixed grid
local rownum=0
foreach yvar in math english {
	foreach y4q in 0 1 2 3 all {
		local ++ rownum
		ipolate yval rank0 if yvar=="`yvar'" & y4quartile=="`y4q'",generate(q0) epolate
		ipolate yval rank1 if yvar=="`yvar'" & y4quartile=="`y4q'",generate(q1) epolate
		mkmat q0 if yvar=="`yvar'" & y4quartile=="`y4q'" & forgrid==1
		mkmat q1 if yvar=="`yvar'" & y4quartile=="`y4q'" & forgrid==1
		mat formatlab[`rownum',1]=q0'
		mat formatlab[`rownum',`ngrid'+1]=q1'
		drop q0 q1
	}
}
clear
svmat formatlab
export delimited formatlab* using `output'formatlab.csv, novarnames nolabel replace
drop formatlab*

pause Run matlab script replication_kipp.m, then type "q"
* now load matlab results
import delimited using `output'frommatlab.csv,varnames(nonames) clear
rename (v1 v2 v3) (deltaval deltacdflb deltacdfub)

* define weights
local math1w=.2382346
local math2w=.2213613
local math3w=.2685376
local math4w=.2718664

gen yvar=""
gen y4quartile=""

local rownum=0
local numts=50
foreach yvar in math {
	foreach y4q in 0 1 2 3 all {
		local ++ rownum
		local first=(`rownum'-1)*`numts'+1
		local last=`first'+`numts'-1
		replace yvar="`yvar'" in `first'/`last'
		replace y4quartile="`y4q'" in `first'/`last'
	}

}

reshape wide deltacdflb deltacdfub, i(yvar deltaval) j(y4quartile) string
gen deltacdfubweighted=.
gen deltacdflbweighted=.
foreach yvar in math {
	replace deltacdfubweighted=``yvar'1w'*deltacdfub0+``yvar'2w'*deltacdfub1+``yvar'3w'*deltacdfub2+``yvar'4w'*deltacdfub3 if yvar=="`yvar'"
	replace deltacdflbweighted=``yvar'1w'*deltacdflb0+``yvar'2w'*deltacdflb1+``yvar'3w'*deltacdflb2+``yvar'4w'*deltacdflb3 if yvar=="`yvar'"
}

* save results for inclusion in graphs for paper

save replication_kippglobal.dta,replace

use `goback'

* A: keep only analysis sample, that is, observations for which outcomes and covariates are non-missing
foreach var of varlist `y1'  `y10' `d' `z' `covariates' {
	keep if `var'!=.
}



* Table 1: summary statistics and covariate balance
sum `y1' `y10' `d' `z' `covariates',detail
sum `y1' `y10' `d' `z' `covariates' if `z'==0,detail
sum `y1' `y10' `d' `z' `covariates' if `z'==1,detail

* joint balance test
local sulist
foreach var of varlist `y10'  `covariates' {
	qui reg `var' `z'
	estimates store `var'reg
	local sulist `sulist' `var'reg
}
suest `sulist',vce(robust)
test `z'


* Table 2: local average effects and first stage
ivregress 2sls `y1' (`d'=`z') `covariates',first vce(robust)

* Table 3: test of stochastic increasingness 

foreach y in `y1' {
	mkspline sspline=`y'4,cubic nknots(`nknots')
	foreach snum in 1 2 3 {
		local s `s`snum''
		display "predictor set in this iteration: `s'"
		* predict Y(0)|S
		reg `y' `s' if `z'==0
		predict `y'0hat
		local r20=e(r2)
		* predict Y(1)|S
		reg `y' `s' if `z'==1
		predict `y'1hat
		local r21=e(r2)
		
		
		* first estimate instrument propensity score
		logit `z' `s'
		predict pz
		display "d = `d'"
		gen kappa0=(1-`d')*((1-`z')-(1-pz))/(1-pz)/pz
		gen kappa1=`d'*(`z'-pz)/(1-pz)/pz
		gen kappa=1-`d'*(1-`z')/(1-pz)-(1-`d')*`z'/pz
				
		
		* generate y0hatkappa,y1hatkappa
		foreach dval in 0 1 {
			sum kappa`dval'
			local kappa`dval'sum=r(N)*r(mean)
			gen `y'kappa`dval'=`y'*kappa`dval'
			sum `y'kappa`dval'
			local `y'kappa`dval'sum=r(N)*r(mean)
			local `y'kappa`dval'mean=``y'kappa`dval'sum'/`kappa`dval'sum'


			mata output=wls("`y' kappa`dval' `s'","`s'")
			mata st_matrix("beta",*output[1])
			mat li beta	
			mata st_matrix("`y'`dval'hatkappa",*output[3])
			svmat `y'`dval'hatkappa
			
			* calculate kappa-weighted r squared
			gen denomvar`dval'=(`y'-``y'kappa`dval'mean')^2*kappa`dval'
			gen numervar`dval'=(`y'`dval'hatkappa-``y'kappa`dval'mean')^2*kappa`dval'
			sum numervar`dval'
			local numer`dval'=r(mean)
			sum denomvar`dval'
			local denom`dval'=r(mean)
			local r2`dval'=`numer`dval''/`denom`dval''
		}
		
		* calculate pearson corr between hats
		gen numervar= (`y'0hatkappa-``y'kappa0mean')*(`y'1hatkappa-``y'kappa1mean')*kappa
		sum numervar
		local numer=r(mean)
		foreach dval in 0 1 {
			gen denom`dval'var=(`y'`dval'hatkappa-``y'kappa`dval'mean')^2*kappa
			sum denom`dval'var
			local denom`dval'=r(mean)
		}
		
		local denom=sqrt(`denom0'*`denom1')
		local rho=`numer'/`denom'
		* compare to function of r2s:
		local rhs=sqrt((1-`r20')*(1-`r21')/`r20'/`r21')
		
		display "test of SI (with kappa-weighting), y = `y', s = `s': `rho' > = `rhs'"


		drop pz `y'1hatkappa `y'0hatkappa `y'kappa0 `y'kappa1 kappa0 kappa1 kappa numervar denomvar1 denomvar0 numervar1 numervar0 denom0var denom1var
	}
	
	drop sspline*
}

* Tables 4-5: bounds on unconditional and conditional distribution of treatment effects

* make spline of baseline score

local s1 ones
local s2 sspline*
local s3 sspline*
local covs3 `covariates'
local s1title Predictors: none
local s2title Predictors: baseline outcome
local s3title Predictors: baseline outcome and covariates


foreach y in `y1' {
	* choose grid of TE values and Y(0) values
	foreach dval in 0 1 {
		sum `y' if `d'==`dval'
		local max`dval'=r(max)
		local min`dval'=r(min)
	}
	local t1=`max1'-`min0'
	local t0=`min1'-`max0'
	local gridn=25
	local tstep=(`t1'-`t0')/`gridn'
	
	numlist "0 `t0'(`tstep')`t1'"
	local telist `r(numlist)'
	sum `y' if `d'==0,detail
	local y0values
	foreach j in 10 25 50 75 90 {
		local next=r(p`j')
		local y0values `y0values' `next'
	}
	
	foreach snum in 3 1 2 {
		preserve
		mkspline sspline=`y'4,cubic nknots(`nknots')
		tedistboundscicentered `y' `d' `z' `s`snum'',strata(`covs`snum'') tevalues(`telist') y0values(`y0values') wd
		mat deltacdfci=r(deltacdfci)
		mat deltacdfwdci=r(deltacdfwdci)
		mat cateci=r(cateci)
		mat cdeltacdfci=r(cdeltacdfci)
		local cdfvars tevalue cdflb cdfub impercentile cdflbse cdfubse cdflbbsl cdflbbsu cdfubbsl cdfubbsu
		local wdcdfvars tevaluewd cdfwdlb cdfwdub impercentilewd cdfwdlbse cdfwdubse cdfwdlbbsl cdfwdlbbsu cdfwdubbsl cdfwdubbsu
		local catevars `y'0value catelb cateub impercentilecate catelbse cateubse catelbbsl catelbbsu cateubbsl cateubbsu
		local cdeltacdfcivars `y'0valueccdf tevalueccdf ccdflb ccdfub impercentileccdf ccdflbse ccdfubse ccdflbbsl ccdflbbsu ccdfubbsl ccdfubbsu
		svmat deltacdfci
		rename (deltacdfci*) (`cdfvars')
		svmat deltacdfwdci
		rename (deltacdfwdci*) (`wdcdfvars')
		svmat cateci
		rename (cateci*) (`catevars')
		svmat cdeltacdfci
		rename (cdeltacdfci*) (`cdeltacdfcivars')
		keep `cdfvars' `wdcdfvars' `catevars' `cdeltacdfcivars'
		save `output'`y'`snum'results.dta,replace
		restore
	}
}

* make figures used in paper
local xtitlepart untreated
local titlepart Untreated

local percentilevarcdf impercentile
local percentilevarcdfwd impercentilewd
local percentilevarcate impercentilecate
local percentilevarccdf impercentileccdf
local evalvarcdf tevalue
local evalvarcdfwd tevaluewd


local alpha=`coverage'
local appendlist
*foreach snum in 3 2 1 {
foreach snum in 3 1 {
	foreach y in math {	
		local evalvarcate `y'0value 
		local evalvarccdf `y'0valueccdf
		*use `cateses' if source=="`y'`snum'",clear
		*tempfile correctedcate`y'`snum'
		*save "`correctedcate`y'`snum''"
		use kipp_replication`y'`snum'results.dta,clear
		
		* create the different CIs
		foreach param in cdf cdfwd ccdf cate {
			
			replace  `percentilevar`param''=`alpha' if `percentilevar`param''==.
			* naive percentile CIs
			gen `param'cinaivelb=`param'lbbsl
			gen `param'cinaiveub=`param'ubbsu
			* centered percentile CIs
			gen `param'cicenteredlb=2*`param'lb-`param'lbbsu
			gen `param'cicenteredub=2*`param'ub-`param'ubbsl
			* SE CIs
			gen `param'critval=invnormal(`percentilevar`param'')
			gen `param'ciselb=`param'lb-`param'critval*`param'lbse
			gen `param'ciseub=`param'ub+`param'critval*`param'ubse
			
			if "`param'"!="cate" {
				foreach var in cinaivelb cinaiveub cicenteredlb cicenteredub ciselb ciseub {
					replace `param'`var'=max(0,min(1,`param'`var'))
				}
			}
			
			sort `evalvar`param''
			
		}
		tempfile ests`y'`snum'
		
		keep tevalue tevaluewd `y'0value cdflb cdfub cdfwdlb cdfwdub catelb cateub cateciselb cateciseub ///
		cdfciselb cdfciseub cdfwdciselb cdfwdciseub `y'0valueccdf tevalueccdf ccdflb ccdfub ccdfciselb ccdfciseub 
		gen source="`y'`snum'"
		save `ests`y'`snum''
		local appendlist `appendlist' `ests`y'`snum''
	}
}	
clear

* this appends results from Matlab implementation of uniform bounds
append using `appendlist' replication_kippglobal


* now make graphs for the papers

* Figure 6: ATE conditional on Y(0)
local y math
sort `y'0value
twoway (line cateub `y'0value if source=="`y'3",lcolor(black) lpattern(solid)) ///
	   (line catelb `y'0value if source=="`y'3",lcolor(black) lpattern(solid)) ///
	   (line cateciseub `y'0value if source=="`y'3",lcolor(black) lpattern(dash)) ///
	   (line cateciselb `y'0value if source=="`y'3",lcolor(black) lpattern(dash)) ///
		,title("Bounds on Average Effect on ``y'title'" ///
		"Conditional on Untreated Potential ``y'title'") ///
		xtitle("``y'title' in untreated state") ytitle("") ///
		legend(label(1 "estimated bounds") ///
				label(3 "95-percent confidence bands") order(1 3)) 
				
graph export `output'`y'13atexyci`r'.pdf,as(pdf) replace

* Figure 5: conditional CDF given Y(0) evaluated at zero
local y math
sort `y'0valueccdf

twoway (line ccdfub `y'0valueccdf if source=="`y'3" & tevalueccdf==0,lcolor(black) lpattern(solid)) ///
	   (line ccdflb `y'0valueccdf if source=="`y'3" & tevalueccdf==0,lcolor(black) lpattern(solid)) ///
	   (line ccdfciseub `y'0valueccdf if source=="`y'3" & tevalueccdf==0,lcolor(black) lpattern(dash)) ///
	   (line ccdfciselb `y'0valueccdf if source=="`y'3" & tevalueccdf==0,lcolor(black) lpattern(dash)) ///
		,title("Bounds on Probability of Negative Effect on ``y'title'" ///
		"Conditional on Untreated Potential ``y'title'") ///
		xtitle("``y'title' in untreated state") ytitle("") ///
		legend(label(1 "estimated bounds") ///
				label(3 "95-percent confidence bands") order(1 3)) 
				
graph export `output'`y'13ccdfxyci`r'.pdf,as(pdf) replace

sort tevalue deltaval

* Figure 2: TE CDF: WD vs SI pointwise vs SI uniform (no covariates)
twoway (line cdfwdub cdfwdlb tevaluewd if source=="`y'1",lpattern(solid solid) lcolor(black black)) ///
	   (line cdfub tevalue if source=="`y'1",lpattern(shortdash) lcolor(black)) ///
	   (line cdflb tevalue if source=="`y'1",lpattern(shortdash) lcolor(black)) ///
       (line deltacdflball deltacdfuball deltaval if yvar=="`y'" & float(deltaval)!=float(1.9168),lcolor(black black) lpattern(dash dash)) ///
		,title("Bounds on CDF of Effect on ``y'title'") ///
			xtitle("effect") ytitle("") ///
			legend(label(1 "Williamson-Downs") ///
			label(3 "Stochastic increasingness" "(no covariates," "integrated pointwise)") ///
			label(5 "Stochastic increasingness" "(no covariates," "uniform)")  order(1 - "" 3 5) ) 
		graph export `output'`y'13deltacdfglobal.pdf,as(pdf) replace

*Figure 3: TE CDF: SI with and without covs 
twoway (line cdfub tevalue if source=="`y'1",lpattern(solid) lcolor(black)) ///
	   (line cdflb tevalue if source=="`y'1",lpattern(solid) lcolor(black)) ///
  	   (line cdfub tevalue if source=="`y'3",lpattern(dash) lcolor(black)) ///
	   (line cdflb tevalue if source=="`y'3",lpattern(dash) lcolor(black)) ///
		,title("Bounds on CDF of Effect on ``y'title'") subtitle("Role of covariates") ///
		xtitle("effect") ytitle("") ///
		legend(label(1 "no covariates") ///
		label(3 "with covariates") order(1 3) )  xlabel(-5(1)5)
		
		graph export `output'`y'13deltacdf.pdf,as(pdf) replace
		
* Figure 4: TE CDF: SI with covariates and CIs
twoway (line cdfub tevalue if source=="`y'3",lpattern(solid) lcolor(black)) ///
	   (line cdflb tevalue if source=="`y'3",lpattern(solid) lcolor(black)) ///
  	   (line cdfciseub tevalue if source=="`y'3",lpattern(dash) lcolor(black)) ///
	   (line cdfciselb tevalue if source=="`y'3",lpattern(dash) lcolor(black)) ///
		,title("Bounds on CDF of Effect on ``y'title'") subtitle("with confidence bands") ///
		xtitle("effect") ytitle("") ///
		legend(label(1 "estimated bounds") ///
		label(3 "95-percent confidence bands") order(1 3) )  xlabel(-5(1)5)
		
		graph export `output'`y'3deltacdfci.pdf,as(pdf) replace

		
* now list values for tables

list cdfwdciselb cdfwdlb cdfwdub cdfwdciseub cdfciselb cdflb cdfub cdfciseub  if tevalue==0 & source=="`y'1"
list deltacdflball deltacdfuball  if yvar=="`y'" & deltaval==0
list cdfwdciselb cdfwdlb cdfwdub cdfwdciseub cdfciselb cdflb cdfub cdfciseub  if tevalue==0 & source=="`y'3"
sort `y'0valueccdf
list `y'0valueccdf ccdflb ccdfub ccdfciselb ccdfciseub if source=="`y'1" & ccdflb!=. & tevalueccdf==0
list `y'0valueccdf ccdflb ccdfub ccdfciselb ccdfciseub if source=="`y'3" & ccdflb!=. & tevalueccdf==0
sort `y'0value
list `y'0value catelb cateub cateciselb cateciseub if source=="`y'1" & catelb!=.
list `y'0value catelb cateub cateciselb cateciseub if source=="`y'3" & catelb!=.

log close




